unit Bh_util1;    {File utilities. Bernd Hoen, 27.3.1996}

interface

uses SysUtils;

const
    CopyBufferLength : Longint = 16384;

type
  TOverwrite      = (No, Ask, Yes);
  ECopyError      = class(Exception);
  EDiskFullError  = class(Exception);
  EOverwriteError = class(Exception);

Function StripBkSlash(S:String):String;  {Remove last backslash if available}
Function AddBkSlash(S:String):String;    {Add last backslash if not available}
Function ThePath(S:String):String;
  {Function ThePath Returns the path S in the form
     x:\
     x:\aaa
     x:\aaa\bbb
   This is compatible with the DirectoryListBox's property Directory
   and fullfills the need of the FileListBox's property Directory}
procedure CopyFileO (SrcName, DestPath:String; Overwrite:Boolean);
procedure CopyFileOB(SrcName, DestPath:String; Overwrite:Boolean;
                        CopyBuffer:Pointer; CopyBufferLength:Longint);
  {Exceptions:
  ECopyError     ('Destination '+DestPath+' is not a directory.');
  ECopyError     (FmtLoadStr(SFOpenError, [SrcName]));
  EOverwriteError('File '+DestName+' already exists');
  EFCreateError  (FmtLoadStr(SFCreateError, [DestName]));
  }
function HasAttr(const FileName: string; Attr: Word): Boolean;
function GetFileSize(const FileName: string): LongInt;
Function GetHighestDirNumber(Path:string):Longint;
Function GetHighestDirNumberString(Path:string):String;
function DiskInserted(Drive:Char):boolean;
function DeleteFileAndAsk(Filename:string; Overwrite:Boolean):boolean;



implementation {-------------------------------------------------------------}

uses Dialogs, Controls, Classes, Consts, WinProcs, WinTypes;

Function StripBkslash(S:String):String;
  begin
  if S[Length(S)]='\' then
    System.Delete(S,Length(S),1);
  Result:=S;
  end;

Function AddBkslash(S:String):String;
  begin
  if S[Length(S)]<>'\' then
    if Length(S)<255 then
      S:=S+'\';
  Result:=S;
  end;

Function ThePath(S:String):String;
  {Function ThePath Returns the path in the form
     x:\
     x:\aaa
     x:\aaa\bbb
   This is compatible with the DirectoryListBox's property Directory
   and fullfills the need of the FileListBox's property Directory}
  begin
  StripBkSlash(S);
  if length(S)=2 then AddBkSlash(S);
  Result:=S;
  end;


procedure CopyFileO(SrcName, DestPath: String; Overwrite:boolean);
  var
    CopyBuffer       : Pointer;
  begin
  GetMem(CopyBuffer, CopyBufferLength);    { allocate the buffer }
  try
    CopyFileOB(SrcName, DestPath, Overwrite, CopyBuffer, CopyBufferLength);
  finally
    FreeMem(CopyBuffer, CopyBufferLength); { free the buffer }
  end{try};
  end{procedure CopyFileO};


procedure CopyFileOB(SrcName, DestPath:String; Overwrite:Boolean;
                        CopyBuffer:Pointer; CopyBufferLength:Longint);
  const
    crlf=#$0d;
  var
    DestName:String;
    TimeStamp, BytesCopied: Longint;
    Source, Dest: Integer; { handles }
    Attr:Integer;
  begin
  DestPath := ExpandFileName(DestPath); { expand the destination path }
  DestPath:=StripBkSlash(DestPath);
  if not HasAttr(DestPath, faDirectory) then
    raise ECopyError.Create('Destination '+DestPath+' is not a directory.');
  DestName := DestPath + '\' + ExtractFileName(SrcName); { ...clone file name }
  TimeStamp := FileAge(SrcName); { get source's time stamp }
  Source := FileOpen(SrcName, fmShareDenyWrite); { open source file }
  if Source < 0 then
    raise ECopyError.Create(FmtLoadStr(SFOpenError, [SrcName]));
  try{source file is open}

    if FileExists(DestName) then
      if not Overwrite then
        raise EOverwriteError.Create('File '+DestName+' already exists')
      else
        begin {remove write protection}
        Attr:=FileGetAttr(DestName);
        if (Attr and faReadOnly)<>0 then
          FileSetAttr(DestName, Attr and not faReadOnly);
        end;
    Dest := FileCreate(DestName); { create output file }
    if Dest < 0 then
      raise EFCreateError.Create(FmtLoadStr(SFCreateError, [DestName]));
    try
      repeat
        BytesCopied := FileRead(Source, CopyBuffer^, CopyBufferLength); { read chunk }
        if BytesCopied > 0 then { if we read anything... ...write chunk }
          if FileWrite(Dest, CopyBuffer^, BytesCopied)<>BytesCopied then
            raise EDiskFullError.Create('Disk full');
      until BytesCopied < CopyBufferLength; { until we run out of chunks }
    finally
      FileSetDate(Dest, TimeStamp);
      FileClose(Dest); { close the destination file }
      {SetFileTimeStamp(Destination, TimeStamp);} { clone source's time stamp }{!!!}
    end;

  finally
    FileClose(Source); { close the source file }
  end{try source file is open};

  end{procedure CopyFileOB};




function HasAttr(const FileName: string; Attr: Word): Boolean;
begin
  Result := (FileGetAttr(FileName) and Attr) = Attr;
end;


function GetFileSize(const FileName: string): LongInt;
  var
    SearchRec: TSearchRec;
  begin
  if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
  else Result := -1;
  FindClose(SearchRec);
  end;


Function GetHighestDirNumber(Path:string):Longint;
  var
    SearchRec: TSearchRec;
    ThisNumber :Longint;
    RetVal   :integer;
    DirName  :string;
  begin
  Path:=AddBkSlash(Path)+'*.*';
  Result:=0;
  RetVal:=FindFirst(Path, faAnyFile, SearchRec);
  while RetVal=0 do
      begin
      if (SearchRec.Attr and faDirectory)>0 then
        begin
        DirName:=SearchRec.Name;
        ThisNumber:=StrToIntDef(DirName, 0); {handles '..' too}
        if ThisNumber>Result then
          Result:=ThisNumber;
        end;
      RetVal:=FindNext(SearchRec);
      end;
  FindClose(SearchRec);
  end;


Function GetHighestDirNumberString(Path:string):String;
  var
    Number:Longint;
  begin
  Number:=GetHighestDirNumber(Path);
  inc(Number);
  if Number>=1000 then
    begin
    MessageDlg('Can''t create directory name with numbers greater than 999',
                mtInformation, [mbOK], 0);
    Result:='';
    exit;
    end;
  Result:=Format('%3.3d', [Number]);
  end;


function DiskInserted(Drive:Char):boolean;
  var
    DriveNum  : byte;
    ErrorMode : word;
  begin
    case Drive of
      'A'..'Z':DriveNum:=Ord(Drive)-Ord('A')+1;
      'a'..'z':DriveNum:=Ord(Drive)-Ord('a')+1;
      end;
    ErrorMode := SetErrorMode(SEM_FailCriticalErrors);
    try
      if DiskSize(DriveNum) = -1 then
        begin
        ShowMessage('Drive not ready');
        Result:=False;
        end
      else
        Result:=True;
    finally
      SetErrorMode(ErrorMode);
    end;
  end;
  {This code came from Lloyd's help file!}


function DeleteFileAndAsk(Filename:string; Overwrite:Boolean):boolean;
  var
    Attr : integer;
  begin
  Result:=False;
  If FileExists(Filename) then
    begin {check for readonly, (ask for) remove it and try again}
    Attr:=FileGetAttr(Filename);
    if Attr<0 then exit;
    if (Attr and faReadOnly)<>0 then
      begin
      if Overwrite then
        begin
        if FileSetAttr(Filename, Attr and not faReadOnly) <> 0 then exit;
        end
      else if MessageDlg('File is write protected.' + #13 +
                         'Remove protection and delete file ?',
                          mtConfirmation, [mbYes, mbNo], 0) = mrYes
        then
          if FileSetAttr(Filename, Attr and not faReadOnly) <> 0 then exit;
      end;
    Result:=DeleteFile(Filename);
    end;
  end;

end.
